<?php

/**
 * @file
 * Contains \Drupal\composer_manager\Composer\Command.
 */

namespace Drupal\composer_manager\Composer;

use Composer\Script\Event;
use Composer\Console\Application;
use Drupal\Component\FileCache\FileCacheFactory;

/**
 * Callbacks for Composer commands defined by Composer Manager.
 *
 * Commands:
 * - 'composer drupal-rebuild'
 * - 'composer drupal-install'
 * - 'composer drupal-update'
 */
class Command {

  /**
   * Rebuilds the root package.
   */
  public static function rebuild(Event $event) {
    $package_manager = self::getPackageManager();
    $package_manager->rebuildRootPackage();

    echo 'The composer.json has been successfuly rebuilt.' . PHP_EOL;
  }

  /**
   * Rebuilds the root package, then calls 'composer install'.
   */
  public static function install(Event $event) {
    $package_manager = self::getPackageManager();
    $package_manager->rebuildRootPackage();

    // Change the requested command to 'install', and rerun composer.
    $command_index = array_search('drupal-install', $_SERVER['argv']);
    $_SERVER['argv'][$command_index] = 'install';
    $application = new Application();
    $application->run();
  }

  /**
   * Rebuilds the root package, then calls 'composer update'.
   */
  public static function update(Event $event) {
    $package_manager = self::getPackageManager();
    $package_manager->rebuildRootPackage();

    // Change the requested command to 'update', and rerun composer.
    $command_index = array_search('drupal-update', $_SERVER['argv']);
    $_SERVER['argv'][$command_index] = 'update';
    $application = new Application();
    $application->run();
  }

  /**
   * Rewrites the autoload.php file, pointing it to the new vendor directory.
   */
  public static function rewriteAutoload(Event $event) {
    $autoload = '<?php' . PHP_EOL;
    $autoload .= '// Generated by composer_manager.' . PHP_EOL;
    $autoload .= "return require __DIR__ . '/vendor/autoload.php';" . PHP_EOL;
    file_put_contents(getcwd() . '/autoload.php', $autoload);
  }

  /**
   * Returns a \Drupal\composer_manager\PackageManager instance.
   */
  public static function getPackageManager() {
    $root = getcwd();
    require $root . '/autoload.php';
    // The module classes aren't in the autoloader at this point.
    require __DIR__ . '/../ExtensionDiscovery.php';
    require __DIR__ . '/../JsonFile.php';
    require __DIR__ . '/../PackageManagerInterface.php';
    require __DIR__ . '/../PackageManager.php';
    require __DIR__ . '/../RootPackageBuilderInterface.php';
    require __DIR__ . '/../RootPackageBuilder.php';

    // YAML discovery in core uses FileCache which is not available.
    FileCacheFactory::setConfiguration(['default' => ['class' => '\Drupal\Component\FileCache\NullFileCache']]);

    $root_package_builder = new \Drupal\composer_manager\RootPackageBuilder($root);
    $package_manager = new \Drupal\composer_manager\PackageManager($root, $root_package_builder);

    return $package_manager;
  }

}
